__revision__ = "$Id:$"

import sys, os, re, subprocess
from stat import ST_MODE
from distutils import sysconfig
from distutils.core import Command
from distutils.dep_util import newer
from distutils.util import convert_path
from distutils import log

# check if Python is called on the first line with this expression
first_line_re = re.compile('^#!.*python[0-9.]*([ \t].*)?$')

class build_latex(Command):

    description = "build latex documents (convert to pdf)"

    user_options = [
        ('build-dir', 'd', "directory to copy pdf to"),
        ('build-temp', 't', "directory to put temporary build by-products"),
        ('force', 'f', "forcibly build everything (ignore file timestamps")
        ]

    boolean_options = ['force']


    def initialize_options (self):
        self.build_dir = None
        self.build_temp = None
        self.latex_documents = None
        self.force = None
        self.outfiles = None

    def finalize_options (self):
        self.set_undefined_options('build',
                                   ('build_lib', 'build_dir'),
                                   ('build_temp', 'build_temp'),
                                   ('force', 'force'))
        self.latex_documents = []
        self.find_latex_document()

    def find_latex_document(self):
        for dir, dirs, files in os.walk('doc'):
            for x in files:
                if not dir.startswith('_') and x.endswith('.tex'):
                    self.latex_documents.append(os.path.join(dir,x))
    

    def get_source_files(self):
        return self.latex_documents

    def run (self):
        if not self.latex_documents:
            return
        self.make_pdf_files()


    def make_pdf_files (self):
        
        outfiles = []
        for x in self.get_source_files():
            x = convert_path(x)
            (dir,name) = os.path.split(x)
            pdfName = (name.rsplit('.',1))[0] + '.pdf'
            outdir = os.path.join(self.build_dir, dir)
            outfile = os.path.join(outdir, pdfName)
            tempdir = os.path.join(self.build_temp, dir)
            self.mkpath(tempdir)
            self.mkpath(outdir)
            outfiles.append(outfile)
    
            if not self.force and not newer(x, outfile):
                log.debug("not converting %s (up-to-date)", outfile)
                continue
            
            log.info("making pdf of %s", x)
            for i in range(0,2):
                process = subprocess.Popen(['pdflatex','-output-directory',tempdir, x], stdout = subprocess.PIPE, stderr = subprocess.STDOUT )
                process.communicate()
            tempfile = os.path.join(tempdir, pdfName)
            self.copy_file(tempfile,outfile)
            
            
            
            
            
    
            



supportrc=dict(framework_install=True, package_name="amuse", allow_build_failures='some')

def use(arg):
    if arg == "package":
        supportrc["framework_install"]=True
    else:
        if arg not in ["system","installed","environment"]:
            warnings.warn(" assuming framework already installed")
        supportrc["framework_install"]=False

def set_package_name(arg):
    supportrc["package_name"]=arg

def set_allow_build_failures(arg):
    if arg=="yes" or (arg==True): arg='some'
    if arg=="no" or (arg==False): arg='none'
    supportrc["allow_build_failures"]=arg



from amuse.rfi.core import legacy_function, LegacyFunctionSpecification
from amuse.community import (
    CodeInterface,
    LiteratureReferencesMixIn,
    StoppingConditionInterface,
    StoppingConditions,
)
from amuse.community.interface.gd import (
    GravitationalDynamics,
    GravitationalDynamicsInterface,
    GravityFieldInterface,
    GravityFieldCode,
)
from amuse.units import nbody_system


class petarInterface(
    CodeInterface,
    LiteratureReferencesMixIn,
    GravitationalDynamicsInterface,
    StoppingConditionInterface,
    GravityFieldInterface
):

    """
    Parallel, Particle-Particle & Particle-Tree & Few-body integration module

    .. [#] Namekata D., et al., 2018, PASJ, 70, 70
    .. [#] Iwasawa M., Tanikawa A., Hosono N., Nitadori K., Muranushi T., Makino J., 2016, PASJ, 68, 54
    .. [#] Iwasawa M., Portegies Zwart S., Makino J., 2015, ComAC, 2, 6
    .. [#] Wang, L., Nitadori, K., Makino, J., 2020, MNRAS, 493, 3398
    .. [#] Wang, L., Iwasawa, M., Nitadori, K., Makino, J., 2020, MNRAS, accepted, ArXiv: 2006.16560 [astro-ph]
    """

    include_headers = ['interface.h']

    def __init__(self, **keyword_arguments):
        CodeInterface.__init__(
            self, name_of_the_worker="petar_worker", **keyword_arguments)
        LiteratureReferencesMixIn.__init__(self)

    @legacy_function
    def get_theta():
        """
        Get theta, the opening angle for building the tree
        """
        function = LegacyFunctionSpecification()
        function.addParameter(
            'theta_for_tree', dtype='float64', direction=function.OUT,
            description=(
                "theta, the opening angle for building the tree:"
                " between 0 and 1"
            )
        )
        function.result_type = 'int32'
        function.result_doc = """
        0 - OK
            the parameter was retrieved
        -1 - ERROR
            could not retrieve parameter
        """
        return function

    @legacy_function
    def set_theta():
        """
        Set theta, the opening angle for building the tree
        """
        function = LegacyFunctionSpecification()
        function.addParameter(
            'theta', dtype='float64', direction=function.IN,
            description=(
                "theta, the opening angle for building the tree:"
                " between 0 and 1"
            )
        )
        function.result_type = 'int32'
        function.result_doc = """
        0 - OK
            the parameter was set
        -1 - ERROR
            could not set parameter
        """
        return function

    @legacy_function
    def get_changeover_rout():
        """
        Get r_out, the changeover radius outer boundary reference for
        switching long/short-range interactions (if zero, auto-determine)
        """
        function = LegacyFunctionSpecification()
        function.addParameter(
            'r_out', dtype='float64', direction=function.OUT,
            description=(
                "r_out, the changeover radius outer boundary reference"
                " for switching long/short-range interactions (if zero,"
                " auto-determine)"
            )
        )
        function.result_type = 'int32'
        function.result_doc = """
        0 - OK
            the parameter was retrieved
        -1 - ERROR
            could not retrieve parameter
        """
        return function

    @legacy_function
    def set_changeover_rout():
        """
        Set r_out, the changeover radius outer boundary reference for
        switching long/short-range interactions (if zero, auto-determine)
        """
        function = LegacyFunctionSpecification()
        function.addParameter(
            'r_out', dtype='float64', direction=function.IN,
            description=(
                "r_out, the changeover radius outer boundary reference"
                " for switching long/short-range interactions (if zero,"
                " auto-determine)"
            )
        )
        function.result_type = 'int32'
        function.result_doc = """
        0 - OK
            the parameter was set
        -1 - ERROR
            could not set parameter
        """
        return function

    @legacy_function
    def get_changeover_ratio():
        """
        Get ratio_r_cut, the changeover radius ratio between the inner and
        outer boundaries
        """
        function = LegacyFunctionSpecification()
        function.addParameter(
            'ratio_r_cut', dtype='float64', direction=function.OUT,
            description=(
                "ratio_r_cut, the changeover radius ratio between the inner"
                " and outer boundaries"
            )
        )
        function.result_type = 'int32'
        function.result_doc = """
        0 - OK
            the parameter was retrieved
        -1 - ERROR
            could not retrieve parameter
        """
        return function

    @legacy_function
    def set_changeover_ratio():
        """
        Set ratio_r_cut, the changeover radius ratio between the inner and
        outer boundaries
        """
        function = LegacyFunctionSpecification()
        function.addParameter(
            'ratio_r_cut', dtype='float64', direction=function.IN,
            description=(
                "ratio_r_cut, the changeover radius ratio between the inner"
                " and outer boundaries"
            )
        )
        function.result_type = 'int32'
        function.result_doc = """
        0 - OK
            the parameter was set
        -1 - ERROR
            could not set parameter
        """
        return function

    @legacy_function
    def get_group_radius():
        """
        Get r_bin, the group detection maximum radius to switch on AR
        (if zero, auto-determine)
        """
        function = LegacyFunctionSpecification()
        function.addParameter(
            'r_bin', dtype='float64', direction=function.OUT,
            description=(
                "r_bin, the group detection maximum radius to switch on AR"
                " (if zero, auto-determine)"
            )
        )
        function.result_type = 'int32'
        function.result_doc = """
        0 - OK
            the parameter was retrieved
        -1 - ERROR
            could not retrieve parameter
        """
        return function

    @legacy_function
    def set_group_radius():
        """
        Set r_bin, the group detection maximum radius to switch on AR
        (if zero, auto-determine)
        """
        function = LegacyFunctionSpecification()
        function.addParameter(
            'r_bin', dtype='float64', direction=function.IN,
            description=(
                "r_bin, the group detection maximum radius to switch on AR"
                " (if zero, auto-determine)"
            )
        )
        function.result_type = 'int32'
        function.result_doc = """
        0 - OK
            the parameter was set
        -1 - ERROR
            could not set parameter
        """
        return function

    @legacy_function
    def get_rsearch_min():
        """
        Get r_search_min, the minimum neighbor searching radius (if zero,
        auto-determine)
        """
        function = LegacyFunctionSpecification()
        function.addParameter(
            'r_search_min', dtype='float64', direction=function.OUT,
            description=(
                "r_search_min, the minimum neighbor searching radius"
                " (if zero, auto-determine)"
            )
        )
        function.result_type = 'int32'
        function.result_doc = """
        0 - OK
            the parameter was retrieved
        -1 - ERROR
            could not retrieve parameter
        """
        return function

    @legacy_function
    def set_rsearch_min():
        """
        Set r_search_min, the minimum neighbor searching radius (if zero,
        auto-determine)
        """
        function = LegacyFunctionSpecification()
        function.addParameter(
            'r_search_min', dtype='float64', direction=function.IN,
            description=(
                "r_search_min, the minimum neighbor searching radius"
                " (if zero, auto-determine)"
            )
        )
        function.result_type = 'int32'
        function.result_doc = """
        0 - OK
            the parameter was set
        -1 - ERROR
            could not set parameter
        """
        return function

    @legacy_function
    def get_tree_step():
        """
        Get dt_soft, the tree time step (if zero, auto-determine)
        """
        function = LegacyFunctionSpecification()
        function.addParameter(
            'dt_soft', dtype='float64', direction=function.OUT,
            description=(
                "dt_soft, the tree time step (if zero, auto-determine)"
            )
        )
        function.result_type = 'int32'
        function.result_doc = """
        0 - OK
            the parameter was retrieved
        -1 - ERROR
            could not retrieve parameter
        """
        return function

    @legacy_function
    def set_tree_step():
        """
        Set dt_soft, the minimum neighbor searching radius (if zero,
        auto-determine)
        """
        function = LegacyFunctionSpecification()
        function.addParameter(
            'dt_soft', dtype='float64', direction=function.IN,
            description=(
                "dt_soft, the tree time step (if zero, auto-determine)"
            )
        )
        function.result_type = 'int32'
        function.result_doc = """
        0 - OK
            the parameter was set
        -1 - ERROR
            could not set parameter
        """
        return function


class petar(GravitationalDynamics, GravityFieldCode):

    def __init__(self, convert_nbody=None, **keyword_arguments):
        self.stopping_conditions = StoppingConditions(self)

        GravitationalDynamics.__init__(
            self,
            petarInterface(**keyword_arguments),
            convert_nbody,
            **keyword_arguments)

    def define_state(self, handler):
        GravitationalDynamics.define_state(self, handler)
        GravityFieldCode.define_state(self, handler)
        self.stopping_conditions.define_state(handler)

    def define_parameters(self, handler):
        GravitationalDynamics.define_parameters(self, handler)
        self.stopping_conditions.define_parameters(handler)

        handler.add_method_parameter(
            "get_eps2",
            "set_eps2",
            "epsilon_squared",
            "smoothing  parameter for gravity calculations",
            default_value=0.0 | nbody_system.length * nbody_system.length
        )

        handler.add_method_parameter(
            "get_changeover_rout",
            "set_changeover_rout",
            "r_out",
            ("changeover radius outer boundary reference for switching"
             " long/short-range interactions (if zero, auto-determine)"),
            default_value=0.0 | nbody_system.length
        )

        handler.add_method_parameter(
            "get_changeover_ratio",
            "set_changeover_ratio",
            "ratio_r_cut",
            "Changeover radius ratio between the inner and outer boundaries",
            default_value=0.1
        )

        handler.add_method_parameter(
            "get_group_radius",
            "set_group_radius",
            "r_bin",
            ("Group detection maximum radius to switch on AR (if zero,"
             " auto-determine)"),
            default_value=0.0 | nbody_system.length
        )

        handler.add_method_parameter(
            "get_rsearch_min",
            "set_rsearch_min",
            "r_search_min",
            "Minimum neighbor searching radius (if zero, auto-determine)",
            default_value=0.0 | nbody_system.length
        )

        handler.add_method_parameter(
            "get_theta",
            "set_theta",
            "theta",
            "Tree opening angle",
            default_value=0.3
        )

        handler.add_method_parameter(
            "get_tree_step",
            "set_tree_step",
            "dt_soft",
            "Tree time step (if zero, auto-determine)",
            default_value=0.0 | nbody_system.time
        )

    def define_methods(self, handler):
        GravitationalDynamics.define_methods(self, handler)
        self.stopping_conditions.define_methods(handler)

        handler.add_method(
            "set_eps2",
            (
                nbody_system.length * nbody_system.length,
            ),
            (
                handler.ERROR_CODE,
            )
        )

        handler.add_method(
            "get_eps2",
            (),
            (
                nbody_system.length * nbody_system.length,
                handler.ERROR_CODE,
            )
        )

        handler.add_method(
            "set_changeover_rout",
            (
                nbody_system.length,
            ),
            (
                handler.ERROR_CODE,
            )
        )

        handler.add_method(
            "get_changeover_rout",
            (),
            (
                nbody_system.length,
                handler.ERROR_CODE,
            )
        )

        handler.add_method(
            "set_group_radius",
            (
                nbody_system.length,
            ),
            (
                handler.ERROR_CODE,
            )
        )

        handler.add_method(
            "get_group_radius",
            (),
            (
                nbody_system.length,
                handler.ERROR_CODE,
            )
        )

        handler.add_method(
            "set_rsearch_min",
            (
                nbody_system.length,
            ),
            (
                handler.ERROR_CODE,
            )
        )

        handler.add_method(
            "get_rsearch_min",
            (),
            (
                nbody_system.length,
                handler.ERROR_CODE,
            )
        )

        handler.add_method(
            "set_tree_step",
            (
                nbody_system.time,
            ),
            (
                handler.ERROR_CODE,
            )
        )

        handler.add_method(
            "get_tree_step",
            (),
            (
                nbody_system.time,
                handler.ERROR_CODE,
            )
        )

    def define_particle_sets(self, handler):
        GravitationalDynamics.define_particle_sets(self, handler)
        self.stopping_conditions.define_particle_set(handler)


PetarInterface = petarInterface
Petar = petar
